/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.notification.util;

import com.sun.jersey.api.client.ClientResponse;
import org.junit.Ignore;
import org.junit.Test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public abstract class BaseTestTemplate extends TestBase {

	protected static RestDescriptor descriptor;

	public BaseTestTemplate () {
	}

	public BaseTestTemplate(RestDescriptor descriptor) {
		this.descriptor = descriptor;
	}

	protected abstract String getArg();
	protected abstract String getInvalidArg();
	protected String error(String body){
		return String.format("%s: %s %s %s", descriptor.getHttpMethod(), descriptor.getPath(), descriptor.getQuery(), body);
	}
	protected void validate20XResponse(ClientResponse response, RestDescriptor descriptor){
		if(response.getStatus() != 204)
			System.out.println(response.getEntity(String.class));
	}
	protected abstract void deleteResource()throws Exception;
	protected abstract void createResource() throws Exception ;
	protected abstract int expectedOkResponseCode();

	protected String getOsduTenantAdminCredentials() throws Exception{
		return testUtils.getAdminToken();
	}

	@Test
	public void should_return401_when_noAccessOnCustomerTenant() throws Exception {
		ClientResponse response = descriptor.runOnCustomerTenant(getArg(), getOsduTenantAdminCredentials());
		assertEquals(error(	response.getEntity(String.class)),401, response.getStatus());
	}

	@Test
	public void should_return401_when_accessingWithAdminCredentials() throws Exception{
		ClientResponse response = descriptor.run(getArg(), testUtils.getAdminToken());
		assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
	}

	@Test
	public void should_return401_when_accessingWithEditorCredentials() throws Exception{
		ClientResponse response = descriptor.run(getArg(), testUtils.getEditorToken());
		assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
	}

	@Test
	public void should_return401_when_accessingWithNoAccessCredentials() throws Exception{
		ClientResponse response = descriptor.run(getArg(), testUtils.getNoAccessToken());
		assertEquals(error(	response.getEntity(String.class)), 401, response.getStatus());
	}

	@Test
	public void should_return20X_when_usingCredentialsWithOpsPermission() throws Exception{
		createResource();

		try {
			ClientResponse response = descriptor.run(getArg(), testUtils.getOpsToken());

			assertEquals(error(response.getStatus() == 204 ? "" : response.getEntity(String.class)), expectedOkResponseCode(), response.getStatus());
			assertEquals("GET, POST, PUT, DELETE, OPTIONS, HEAD, PATCH", response.getHeaders().getFirst("Access-Control-Allow-Methods"));
			assertEquals("access-control-allow-origin, origin, content-type, accept, authorization, data-partition-id, correlation-id, appkey", response.getHeaders().getFirst("Access-Control-Allow-Headers"));
			assertEquals("*", response.getHeaders().getFirst("Access-Control-Allow-Origin"));
			assertEquals("true", response.getHeaders().getFirst("Access-Control-Allow-Credentials"));
			assertEquals("DENY", response.getHeaders().getFirst("X-Frame-Options"));
			assertEquals("1; mode=block", response.getHeaders().getFirst("X-XSS-Protection"));
			assertEquals("nosniff", response.getHeaders().getFirst("X-Content-Type-Options"));
			assertEquals("no-cache, no-store, must-revalidate", response.getHeaders().getFirst("Cache-Control"));
			assertEquals("default-src 'self'", response.getHeaders().getFirst("Content-Security-Policy"));
			assertTrue(response.getHeaders().get("Strict-Transport-Security").get(0).contains("max-age=31536000"));
			assertTrue(response.getHeaders().get("Strict-Transport-Security").get(0).contains("includeSubDomains"));
			assertEquals("0", response.getHeaders().getFirst("Expires"));
		} finally {
			deleteResource();
		}
	}

	@Test
	public void should_returnOk_when_makingHttpOptionsRequest() throws Exception{
		ClientResponse response = descriptor.runOptions(getArg(), testUtils.getEditorToken());
		assertEquals(error(	response.getEntity(String.class)), 200, response.getStatus());
	}
	@Ignore("Issue logged in GL")
	@Test
	public void should_return307_when_makingHttpRequest()throws Exception{
		if(Config.Instance().isLocalHost() || Config.isGke())
			return;	//localhost and GKE we expect to be able to use http so we shortcut the test

		ClientResponse response = descriptor.runHttp(getArg(), testUtils.getOpsToken());
		assertEquals(error(	response.getEntity(String.class)),307, response.getStatus());
	}
	@Test
	public void should_return20XResponseCode_when_makingValidHttpsRequest()throws Exception{
		createResource();
		try {
			ClientResponse response = descriptor.run(getArg(), testUtils.getOpsToken());
			assertEquals(error(""), expectedOkResponseCode(), response.getStatus());
			validate20XResponse(response, descriptor);
		}finally {
			deleteResource();
		}
	}

	@Test
	public void should_return400_when_makingHttpRequestWithoutToken()throws Exception{
		ClientResponse response =  descriptor.run(getArg(), "");
		assertEquals(error(	response.getEntity(String.class)), 302, response.getStatus());
	}

}
