/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.notification.util;

import static org.junit.Assert.assertEquals;

import java.net.SocketTimeoutException;
import java.net.URL;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.*;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public abstract class TestUtils {

    protected String serviceAccountFile;
    protected static String opsToken = null;
    protected static String adminToken = null;
    protected static String editorToken = null;
    protected static String noAccessToken = null;

    public static String getApiPath(String api, boolean enforceHttp) throws Exception {
        String baseUrl = Config.Instance().HostUrl;
        if (enforceHttp)
            baseUrl = baseUrl.replaceFirst("https", "http");
        URL mergedURL = new URL(baseUrl + api);
        return mergedURL.toString();
    }

    public static String getApiPath(String baseUrl, String api, boolean enforceHttp) throws Exception {
        if (enforceHttp)
            baseUrl = baseUrl.replaceFirst("https", "http");
        URL mergedURL = new URL(baseUrl + api);
        return mergedURL.toString();
    }

    public static String getOsduTenant() {
        return Config.Instance().OsduTenant;
    }

    public static String getCustomerTenant() {
        return Config.Instance().ClientTenant;
    }

    public abstract String getOpsToken() throws Exception;

    public abstract String getAdminToken() throws Exception;

    public abstract String getEditorToken() throws Exception;

    public abstract String getNoAccessToken() throws Exception;

    public static ClientResponse send(String path, String httpMethod, String token, String requestBody, String query,
                                      Map<String, String> headers, boolean enforceHttp)
            throws Exception {
        ClientResponse response = null;
        Client client = getClient();
        client.setConnectTimeout(300000);
        client.setReadTimeout(300000);
        client.setFollowRedirects(false);
        String url = getApiPath(path + query, enforceHttp);
        WebResource webResource = client.resource(url);
        int count = 1;
        int MaxRetry = 3;
        while (count < MaxRetry) {
            try {
                headers.put("correlation-id", headers.getOrDefault("correlation-id", UUID.randomUUID().toString()));
                WebResource.Builder builder = webResource.type(MediaType.APPLICATION_JSON)
                        .header("Authorization", token);
                headers.forEach((k, v) -> builder.header(k, v));
                //removing Auth header before logging
                headers.remove("Authorization");
                log.info(String.format("\nRequest URL: %s %s\nRequest Headers: %s\nRequest Body: %s", httpMethod, url, headers, requestBody));
                log.info(String.format("Attempt: #%s/%s, CorrelationId: %s", count, MaxRetry, headers.get("correlation-id")));
                response = builder.method(httpMethod, ClientResponse.class, requestBody);
                if (response.getStatusInfo().getFamily().equals(Response.Status.Family.valueOf("SERVER_ERROR"))) {
                    count++;
                    Thread.sleep(5000);
                    continue;
                } else {
                    break;
                }
            } catch (Exception ex) {
                log.error("Exception While Making Request: ", ex);
                count++;
                if (count == MaxRetry) {
                    throw new AssertionError("Error: Send request error", ex);
                }
            } finally {
                //log response body
                log.info("sending response from TestUtils send method");
                if(response!=null)
                    log.info(String.format("\nThis is the response received : %s\nResponse Headers: %s\nResponse Status code: %s", response, response.getHeaders(), response.getStatus()));
            }
        }
        return response;
    }

    public static ClientResponse send(String url, String path, String httpMethod, String token, String requestBody,
                                      String query, Map<String, String> headers, boolean enforceHttp)
            throws Exception {
        ClientResponse response = null;
        Client client = getClient();
        client.setConnectTimeout(300000);
        client.setReadTimeout(300000);
        client.setFollowRedirects(false);
        String URL = getApiPath(url, path + query, enforceHttp);
        WebResource webResource = client.resource(URL);
        int count = 1;
        int MaxRetry = 3;
        while (count < MaxRetry) {
            try {
                headers.put("correlation-id", headers.getOrDefault("correlation-id", UUID.randomUUID().toString()));
                WebResource.Builder builder = webResource.type(MediaType.APPLICATION_JSON);
                if (!token.isEmpty()) {
                    log.info("Token is not empty so adding to request header");
                    builder.header("Authorization", token);
                }
                headers.forEach((k, v) -> builder.header(k, v));
                //removing Auth header before logging
                headers.remove("Authorization");
                log.info(String.format("\nRequest URL: %s %s\nRequest Headers: %s\nRequest Body: %s", httpMethod, URL, headers, indentatedBody(requestBody)));
                log.info(String.format("Attempt: #%s/%s, CorrelationId: %s", count, MaxRetry, headers.get("correlation-id")));
                response = builder.method(httpMethod, ClientResponse.class, requestBody);
                if (response.getStatusInfo().getFamily().equals(Response.Status.Family.valueOf("SERVER_ERROR"))) {
                    count++;
                    Thread.sleep(5000);
                    continue;
                } else {
                    break;
                }
            } catch (Exception ex) {
                log.error("Exception While Making Request: ", ex);
                count++;
                if (count == MaxRetry) {
                    throw new AssertionError("Error: Send request error", ex);
                }
            } finally {
                //log response body
                log.info("sending response from TestUtils send method");
                if(response!=null)
                    log.info(String.format("\nThis is the response received : %s\nResponse Headers: %s\nResponse Status code: %s", response, response.getHeaders(), response.getStatus()));
            }
        }
        return response;
    }

    public static String indentatedBody(String responseBody) {
        JsonParser jsonParser = new JsonParser();
        if( responseBody== null)
            return responseBody;
        JsonElement jsonElement = jsonParser.parse(responseBody);
        String indentedResponseEntity =new GsonBuilder().setPrettyPrinting().create().toJson(jsonElement);
        return indentedResponseEntity;
    }

    @SuppressWarnings("unchecked")
    public <T> T getResult(ClientResponse response, int exepectedStatus, Class<T> classOfT) {
        String json = response.getEntity(String.class);

        assertEquals(exepectedStatus, response.getStatus());
        if (exepectedStatus == 204) {
            return null;
        }

        assertEquals(MediaType.APPLICATION_JSON, response.getType().toString());
        if (classOfT == String.class) {
            return (T) json;
        }

        Gson gson = new Gson();
        return gson.fromJson(json, classOfT);
    }

    public static Client getClient() {
        TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return null;
            }

            @Override
            public void checkClientTrusted(X509Certificate[] certs, String authType) {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] certs, String authType) {
            }
        }};

        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
            log.info("Exception occurred");
        }
        log.info("Creating client");
        return Client.create();
    }
}